@@ -2,8 +2,7 @@
 //!
 //! A Kalman filter (estimator) with one state and constant noises.
 
-use na::{Matrix1, Vector1};
-use nalgebra as na;
+use nalgebra::{Matrix1, Vector1};
 
 use bayes_estimate::models::{
     ExtendedLinearObserver, ExtendedLinearPredictor, KalmanState
