@@ -25,11 +25,9 @@
 //!   likelihoods have a large range. Resampling becomes ill conditioned for these samples.
 
 
-use na::{DefaultAllocator, Const, Dim, U1, OMatrix, OVector, RealField};
-use na::allocator::Allocator;
-use na::storage::Storage;
-use nalgebra as na;
-use num_traits::{Pow, ToPrimitive};
+use nalgebra::{Const, DefaultAllocator, Dim, OMatrix, OVector, RealField, U1};
+use nalgebra::allocator::Allocator;
+use num_traits::{Pow, ToPrimitive, FromPrimitive};
 use num_traits::real::Real;
 use rand_core::RngCore;
 use rand_distr::{Distribution, StandardNormal, Uniform};
@@ -148,13 +146,13 @@ where
 
 }
 
-impl<N: RealField, D: Dim> Estimator<N, D> for SampleState<N, D>
+impl<N: FromPrimitive + RealField, D: Dim> Estimator<N, D> for SampleState<N, D>
     where
         DefaultAllocator: Allocator<N, D>,
 {
     fn state(&self) -> Result<OVector<N, D>, &'static str> {
         // Mean of distribution: mean of particles
-        let s_shape = self.s[0].data.shape();
+        let s_shape = self.s[0].shape_generic();
         let mut x = OVector::zeros_generic(s_shape.0, s_shape.1);
         for s in self.s.iter() {
             x += s;
@@ -345,11 +343,11 @@ pub fn live_samples<N: RealField, D:Dim>(s: &mut Samples<N, D>, resamples: &Resa
 ///
 /// Numerics:
 ///  If the are very few unique samples the roughening will colapse as it is not representative of the true state distribution.
-pub fn roughen_minmax<N: RealField, D:Dim>(s: &mut Samples<N, D>, k: f32, rng: &mut dyn RngCore)
+pub fn roughen_minmax<N: Copy + FromPrimitive + RealField, D:Dim>(s: &mut Samples<N, D>, k: f32, rng: &mut dyn RngCore)
 where
     DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>,
 {
-    let x_dim = s[0].data.shape().0;
+    let x_dim = s[0].shape_generic().0;
     let x_size = x_dim.value();
 
     // Find min and max state dimension in all states
@@ -382,7 +380,7 @@ where
 /// The roughening is scaled by the deviation of the noise.
 ///
 /// 'k' is scaling factor for normally distributed noise
-pub fn roughen_noise<N: RealField, D: Dim, ND: Dim>(s: &mut Samples<N, D>, noise: &CoupledNoise<N, D, ND>, k: f32, rng: &mut dyn RngCore)
+pub fn roughen_noise<N: Copy + FromPrimitive + RealField, D: Dim, ND: Dim>(s: &mut Samples<N, D>, noise: &CoupledNoise<N, D, ND>, k: f32, rng: &mut dyn RngCore)
     where
         DefaultAllocator: Allocator<N, D, D> + Allocator<N, D, ND> + Allocator<N, ND, ND> + Allocator<N, D> + Allocator<N, ND>
 {
@@ -399,7 +397,7 @@ pub fn roughen_noise<N: RealField, D: Dim, ND: Dim>(s: &mut Samples<N, D>, noise
     }
 }
 
-impl<N: RealField, D: Dim> KalmanEstimator<N, D> for SampleState<N, D>
+impl<N: Copy + FromPrimitive + RealField, D: Dim> KalmanEstimator<N, D> for SampleState<N, D>
     where
         DefaultAllocator: Allocator<N, D, D> + Allocator<N, U1, D> + Allocator<N, D>,
 {
@@ -419,7 +417,7 @@ impl<N: RealField, D: Dim> KalmanEstimator<N, D> for SampleState<N, D>
     fn kalman_state(&self) -> Result<KalmanState<N, D>, &'static str> {
         let x = self.state()?;
         // Covariance of distribution: covariance of particles
-        let s_shape = self.s[0].data.shape();
+        let s_shape = self.s[0].shape_generic();
         let mut xx = OMatrix::zeros_generic(s_shape.0, s_shape.0);
 
         for s in self.s.iter() {
@@ -436,7 +434,7 @@ impl<N: RealField, D: Dim> KalmanEstimator<N, D> for SampleState<N, D>
 /// Generate as sampling function for normally distributed noise.
 ///
 /// 'std_dev' standard deviation of normally distributed noise.
-pub fn normal_noise_sampler<N: RealField, D: Dim>(std_dev: OVector<N, D>) -> impl Fn(&mut dyn RngCore) -> OVector<N,D>
+pub fn normal_noise_sampler<N: Copy + FromPrimitive + RealField, D: Dim>(std_dev: OVector<N, D>) -> impl Fn(&mut dyn RngCore) -> OVector<N,D>
     where
         DefaultAllocator: Allocator<N, D>,
 {
@@ -446,24 +444,24 @@ pub fn normal_noise_sampler<N: RealField, D: Dim>(std_dev: OVector<N, D>) -> imp
             .enumerate()
             .map(|(i, n): (usize, f32)| {N::from_f32(n).unwrap() * std_dev[i]}).
             take(std_dev.nrows());
-        OVector::from_iterator_generic(std_dev.data.shape().0, Const::<1>, rnormal)
+        OVector::from_iterator_generic(std_dev.shape_generic().0, Const::<1>, rnormal)
     }
 }
 
 /// Generate as sampling function for normally distributed coupled noise.
-pub fn normal_noise_sampler_coupled<N: RealField, D: Dim, ND: Dim>(coupleing: OMatrix<N, D, ND>) -> impl Fn(&mut dyn RngCore) -> OVector<N,D>
+pub fn normal_noise_sampler_coupled<N: FromPrimitive + RealField, D: Dim, ND: Dim>(coupleing: OMatrix<N, D, ND>) -> impl Fn(&mut dyn RngCore) -> OVector<N,D>
     where
         DefaultAllocator: Allocator<N, D, ND> + Allocator<N, D> + Allocator<N, ND>,
 {
     // Sample with the coupled noise deviation
     move |rng: &mut dyn RngCore| -> OVector<N,D> {
         let rnormal = StandardNormal.sample_iter(rng).map(|n: f32| {N::from_f32(n).unwrap()}).take(coupleing.ncols());
-        let n = OVector::from_iterator_generic(coupleing.data.shape().1, Const::<1>, rnormal);
+        let n = OVector::from_iterator_generic(coupleing.shape_generic().1, Const::<1>, rnormal);
         &coupleing * n
     }
 }
 
-pub fn gaussian_observation_likelihood<'r, N: RealField + ToPrimitive, D: Dim, ZD: Dim>(
+pub fn gaussian_observation_likelihood<'r, N: Copy + RealField + ToPrimitive, D: Dim, ZD: Dim>(
     z: &'r OVector<N, ZD>, h: fn(&OVector<N, D>) -> OVector<N, ZD>,
     noise: &CorrelatedNoise<N, ZD>) -> impl Fn(&OVector<N, D>) -> f32 + 'r
     where
