@@ -6,17 +6,14 @@
 //!
 //! The linear representation can also be used for non-linear system by using linearised forms of the system model.
 
-use nalgebra as na;
-use na::{allocator::Allocator, DefaultAllocator, storage::Storage};
-use na::{Const, Dim, U1, RealField, OMatrix, OVector};
+use nalgebra::{allocator::Allocator, DefaultAllocator};
+use nalgebra::{U1, Const, Dim, DimAdd, DimSum, OMatrix, OVector, RealField};
 
 use crate::linalg::cholesky::UDU;
 use crate::matrix;
-use crate::models::{KalmanEstimator, KalmanState, Estimator};
-use crate::noise::{UncorrelatedNoise, CoupledNoise, CorrelatedNoise};
-use nalgebra::{DimAdd, DimSum};
-use crate::matrix::{check_non_negativ};
-
+use crate::matrix::check_non_negativ;
+use crate::models::{Estimator, KalmanEstimator, KalmanState};
+use crate::noise::{CorrelatedNoise, CoupledNoise, UncorrelatedNoise};
 
 /// UD State representation.
 ///
@@ -37,7 +34,7 @@ pub struct UDState<N: RealField, D: Dim>
     udu: UDU<N>,
 }
 
-impl<N: RealField, D: Dim> UDState<N, D>
+impl<N: Copy + RealField, D: Dim> UDState<N, D>
     where
         DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>,
 {
@@ -75,7 +72,7 @@ impl<N: RealField, D: Dim> UDState<N, D>
             D: DimAdd<QD>,
             DefaultAllocator: Allocator<N, DimSum<D, QD>, U1> + Allocator<N, D, QD> + Allocator<N, QD>
     {
-        let mut scratch = self.new_predict_scratch(noise.q.data.shape().0);
+        let mut scratch = self.new_predict_scratch(noise.q.shape_generic().0);
         self.predict_use_scratch(&mut scratch, x_pred, fx, noise)
     }
 
@@ -98,7 +95,7 @@ impl<N: RealField, D: Dim> UDState<N, D>
     }
 }
 
-impl<N: RealField, D: Dim> Estimator<N, D> for UDState<N, D>
+impl<N: Copy + RealField, D: Dim> Estimator<N, D> for UDState<N, D>
     where
         DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>,
 {
@@ -107,7 +104,7 @@ impl<N: RealField, D: Dim> Estimator<N, D> for UDState<N, D>
     }
 }
 
-impl<N: RealField, D: Dim> KalmanEstimator<N, D> for UDState<N, D>
+impl<N: Copy + RealField, D: Dim> KalmanEstimator<N, D> for UDState<N, D>
     where
         DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>,
 {
@@ -120,7 +117,7 @@ impl<N: RealField, D: Dim> KalmanEstimator<N, D> for UDState<N, D>
         // Factorise X into UD
         let rows = self.UD.nrows();
         self.UD.copy_from(&state.X);
-        let rcond = self.udu.UdUfactor_variant2(&mut self.UD, rows);
+        let rcond = self.udu.UdUfactor(&mut self.UD, rows);
         matrix::check_non_negativ(rcond, "X not PSD")?;
 
         Ok(())
@@ -131,7 +128,7 @@ impl<N: RealField, D: Dim> KalmanEstimator<N, D> for UDState<N, D>
     /// The covariance matrix X is recomposed from U.d.U' in the UD matrix.
     fn kalman_state(&self) -> Result<KalmanState<N, D>, &'static str> {
         // assign elements of common left block of M into X
-        let x_shape = self.x.data.shape().0;
+        let x_shape = self.x.shape_generic().0;
 
         let mut X = self.UD.columns_generic(0, x_shape).into_owned();
         UDU::UdUrecompose(&mut X);
@@ -151,7 +148,7 @@ pub struct CorrelatedFactorNoise<N: RealField, D: Dim>
     pub UD: OMatrix<N, D, D>
 }
 
-impl<N: RealField, D: Dim> CorrelatedFactorNoise<N, D>
+impl<N: Copy + RealField, D: Dim> CorrelatedFactorNoise<N, D>
     where
         DefaultAllocator: Allocator<N, D, D>,
 {
@@ -160,14 +157,14 @@ impl<N: RealField, D: Dim> CorrelatedFactorNoise<N, D>
     pub fn from_correlated(correlated: &CorrelatedNoise<N, D>) -> Result<Self, &'static str> {
         let udu = UDU::new();
         let mut ud: OMatrix<N, D, D> = correlated.Q.clone_owned();
-        let rcond = udu.UdUfactor_variant2(&mut ud, correlated.Q.nrows());
+        let rcond = udu.UdUfactor(&mut ud, correlated.Q.nrows());
         check_non_negativ(rcond, "Q not PSD")?;
 
         Ok(CorrelatedFactorNoise{ UD: ud })
     }
 }
 
-impl<N: RealField, D: Dim> UDState<N, D>
+impl<N: Copy + RealField, D: Dim> UDState<N, D>
     where
         DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>,
 {
@@ -270,7 +267,7 @@ pub struct ObserveScratch<N: RealField, D: Dim>
     pub b: OVector<N, D>,
 }
 
-impl<N: RealField, D: Dim> UDState<N, D>
+impl<N: Copy + RealField, D: Dim> UDState<N, D>
     where
         DefaultAllocator: Allocator<N, D, D> + Allocator<N, D>,
 {
@@ -280,9 +277,9 @@ impl<N: RealField, D: Dim> UDState<N, D>
             DefaultAllocator: Allocator<N, D, QD> + Allocator<N, DimSum<D, QD>, U1>
     {
         // x + qd rows
-        let xqd_size = self.UD.data.shape().1.add(qd);
+        let xqd_size = self.UD.shape_generic().1.add(qd);
         PredictScratch {
-            G: OMatrix::zeros_generic(self.UD.data.shape().0, qd),
+            G: OMatrix::zeros_generic(self.UD.shape_generic().0, qd),
             d: OVector::zeros_generic(xqd_size, Const::<1>),
             dv: OVector::zeros_generic(xqd_size, Const::<1>),
             v: OVector::zeros_generic(xqd_size, Const::<1>),
@@ -290,7 +287,7 @@ impl<N: RealField, D: Dim> UDState<N, D>
     }
 
     pub fn new_observe_scratch(&self) -> ObserveScratch<N, D> {
-        let x_size = self.x.data.shape().0;
+        let x_size = self.x.shape_generic().0;
         ObserveScratch {
             w: OVector::zeros_generic(x_size, Const::<1>),
             a: OVector::zeros_generic(x_size, Const::<1>),
@@ -364,12 +361,12 @@ impl<N: RealField, D: Dim> UDState<N, D>
             D: DimAdd<QD>,
             DefaultAllocator: Allocator<N, DimSum<D, QD>> + Allocator<N, D, QD> + Allocator<N, QD>,
     {
-        let nx_ = self.x.data.shape().0;
+        let nx_ = self.x.shape_generic().0;
         let nx = nx_.value();
-        let nq_ = q.data.shape().0;
+        let nq_ = q.shape_generic().0;
 
         // Augment d with q, UD with G
-        scratch.d.rows_generic_mut(nx, q.data.shape().0).copy_from(q);
+        scratch.d.rows_generic_mut(nx, q.shape_generic().0).copy_from(q);
         scratch.G.copy_from(G);
 
         // U=Fx*U and diagonals retrieved
