@@ -4,7 +4,7 @@
 
 use nalgebra::{allocator::Allocator, DefaultAllocator, Dim, OMatrix, RealField};
 
-pub fn rcond_symetric<N: RealField, R: Dim, C: Dim>(sm: &OMatrix<N, R, C>) -> N
+pub fn rcond_symetric<N: Copy + RealField, R: Dim, C: Dim>(sm: &OMatrix<N, R, C>) -> N
 where
     DefaultAllocator: Allocator<N, R, C>,
 {
