@@ -1,11 +1,9 @@
 #![allow(dead_code)]
 
-use na::base::constraint::{DimEq, SameNumberOfColumns, SameNumberOfRows, ShapeConstraint};
-use na::{allocator::Allocator, DefaultAllocator, U1, U2};
-use na::{Dim, Vector2};
-use na::{OMatrix, RealField};
-use nalgebra as na;
-use nalgebra::{DimName, Dynamic, Const};
+use nalgebra::{allocator::Allocator, DefaultAllocator, U1, U2};
+use nalgebra::{Const, Dim, DimName, Dynamic};
+use nalgebra::{OMatrix, RealField, Vector2};
+use nalgebra::base::constraint::{DimEq, SameNumberOfColumns, SameNumberOfRows, ShapeConstraint};
 
 trait GenricDim<D: Dim> {
     fn dim(&self) -> D;
